<?php

namespace Drupal\cms_content_sync\Controller;

use Drupal\cms_content_sync\Entity\Flow;
use Drupal\cms_content_sync\Entity\Pool;
use Drupal\cms_content_sync\Plugin\Type\EntityHandlerPluginManager;
use Drupal\cms_content_sync\PullIntent;
use Drupal\cms_content_sync\SyncCoreInterface\DrupalApplication;
use Drupal\Core\Access\AccessResult;
use Drupal\Core\Controller\ControllerBase;

/**
 * Provides a listing of Flow.
 */
class ManualPull extends ControllerBase {

  /**
   * Access check.
   *
   * Ensure that the pull tab is just show if a flow exists which contains
   * and entity type that has its pull set to "manual".
   */
  public function access() {
    $flows = Flow::getAll();
    $manually_pulled_entity_types = [];
    foreach ($flows as $flow) {
      $pull_manually = $flow->getController()->getEntityTypesToPull(PullIntent::PULL_MANUALLY);
      if (!empty($pull_manually)) {
        $manually_pulled_entity_types[$flow->id()] = $pull_manually;
      }
    }

    return AccessResult::allowedIf(!empty($manually_pulled_entity_types))
      ->addCacheableDependency($flows);
  }

  /**
   * Render the content synchronization Angular frontend.
   *
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   * @throws \EdgeBox\SyncCore\Exception\SyncCoreException
   *
   * @return array
   *   Returns the rendered pull dashboard.
   */
  public function content() {
    $settings = ContentSyncSettings::getInstance();

    $embed = Embed::create(\Drupal::getContainer());

    return $embed->pullDashboard();
  }

}
